@@ -1,6 +1,6 @@
 use super::{
     util::{err_modal, toplevel},
-    BSCtx, BSCtxRef, BSNode,
+    BSCtx, BSCtxRef, BSNode, WVal,
 };
 use crate::bscript::LocalEvent;
 use arcstr::ArcStr;
@@ -20,19 +20,17 @@ use netidx::{
     chars::Chars,
     path::Path,
     resolver,
-    subscriber::{Dval, Event, SubId, Typ, UpdatesFlags, Value},
-    utils::split_escaped,
+    subscriber::{Dval, Event, SubId, UpdatesFlags, Value},
 };
 use netidx_bscript::vm;
 use netidx_protocols::view;
+use regex::RegexSet;
 use std::{
     cell::{Cell, RefCell},
-    cmp::Ordering,
+    cmp::{Ordering, PartialEq},
     collections::{HashMap, HashSet},
-    iter::FromIterator,
     rc::{Rc, Weak},
-    result,
-    str::FromStr,
+    result::{self, Result},
 };
 
 struct Subscription {
@@ -69,6 +67,119 @@ struct RaeifiedTable(Rc<RaeifiedTableInner>);
 
 struct RaeifiedTableWeak(Weak<RaeifiedTableInner>);
 
+#[derive(Debug, Clone, PartialEq, Eq)]
+enum SortSpec {
+    None,
+    Ascending(String),
+    Descending(String),
+}
+
+impl SortSpec {
+    fn new(v: Value) -> Result<Self, String> {
+        match v {
+            Value::Null => Ok(SortSpec::None),
+            Value::String(col) => Ok(SortSpec::Descending(String::from(&*col))),
+            Value::Array(a) if a.len() == 2 => {
+                let column =
+                    a[0].clone().cast_to::<String>().map_err(|e| format!("{}", e))?;
+                match a[1]
+                    .clone()
+                    .cast_to::<String>()
+                    .map_err(|e| format!("{}", e))?
+                    .as_str()
+                {
+                    "ascending" => Ok(SortSpec::Ascending(column)),
+                    "descending" => Ok(SortSpec::Descending(column)),
+                    _ => Err("invalid mode".into()),
+                }
+            }
+            _ => Err("expected null, col, or a pair of [column, mode]".into()),
+        }
+    }
+}
+
+#[derive(Debug, Clone)]
+enum Filter {
+    All,
+    Auto,
+    None,
+    Include(HashSet<String>),
+    Exclude(HashSet<String>),
+    IncludeMatch(HashSet<String>, RegexSet),
+    ExcludeMatch(HashSet<String>, RegexSet),
+}
+
+impl PartialEq for Filter {
+    fn eq(&self, other: &Self) -> bool {
+        match (self, other) {
+            (Filter::All, Filter::All)
+            | (Filter::Auto, Filter::Auto)
+            | (Filter::None, Filter::None) => true,
+            (Filter::Include(s0), Filter::Include(s1)) => s0 == s1,
+            (Filter::Exclude(s0), Filter::Exclude(s1)) => s0 == s1,
+            (Filter::IncludeMatch(s0, _), Filter::IncludeMatch(s1, _)) => s0 == s1,
+            (Filter::ExcludeMatch(s0, _), Filter::ExcludeMatch(s1, _)) => s0 == s1,
+            (Filter::All, _)
+            | (_, Filter::All)
+            | (Filter::Auto, _)
+            | (_, Filter::Auto)
+            | (Filter::None, _)
+            | (_, Filter::None)
+            | (Filter::Include(_), _)
+            | (_, Filter::Include(_))
+            | (Filter::Exclude(_), _)
+            | (_, Filter::Exclude(_))
+            | (Filter::IncludeMatch(_, _), _)
+            | (_, Filter::IncludeMatch(_, _)) => false,
+        }
+    }
+}
+
+impl Filter {
+    fn new(v: Value) -> Result<Self, String> {
+        match v {
+            Value::Null => Ok(Filter::Auto),
+            Value::True => Ok(Filter::All),
+            Value::False => Ok(Filter::None),
+            Value::Array(a) if a.len() == 2 => {
+                let mode =
+                    a[0].clone().cast_to::<Chars>().map_err(|e| format!("{}", e))?;
+                let i = a[1]
+                    .clone()
+                    .flatten()
+                    .map(|v| v.cast_to::<String>().map_err(|e| format!("{}", e)));
+                if &*mode == "include" {
+                    Ok(Filter::Include(i.collect::<Result<HashSet<_, _>, _>>()?))
+                } else if &*mode == "exclude" {
+                    Ok(Filter::Exclude(i.collect::<Result<HashSet<_, _>, _>>()?))
+                } else if &*mode == "include_match" {
+                    let set = i.collect::<Result<HashSet<_>, _>>()?;
+                    let matcher = RegexSet::new(&set).map_err(|e| format!("{}", e))?;
+                    Ok(Filter::IncludeMatch(set, matcher))
+                } else if &*mode == "exclude_match" {
+                    let set = i.collect::<Result<HashSet<_>, _>>()?;
+                    let matcher = RegexSet::new(&set).map_err(|e| format!("{}", e))?;
+                    Ok(Filter::ExcludeMatch(set, matcher))
+                } else {
+                    Err("invalid filter mode".into())
+                }
+            }
+            _ => Err("expected null, true, false, or a pair".into()),
+        }
+    }
+
+    fn is_match(&self, s: &str) -> bool {
+        match self {
+            Filter::All | Filter::Auto => true,
+            Filter::None => false,
+            Filter::Include(set) => set.contains(s),
+            Filter::Exclude(set) => !set.contains(s),
+            Filter::IncludeMatch(_, set) => set.is_match(s),
+            Filter::ExcludeMatch(_, set) => !set.is_match(s),
+        }
+    }
+}
+
 enum TableState {
     Empty,
     Resolving(Path),
@@ -83,17 +194,13 @@ pub(super) struct Table {
     path_expr: BSNode,
     path: RefCell<Option<Path>>,
     default_sort_column_expr: BSNode,
-    default_sort_column: RefCell<Option<String>>,
-    default_sort_column_direction_expr: BSNode,
-    default_sort_column_direction: RefCell<SortDir>,
-    column_mode_expr: BSNode,
-    column_mode: RefCell<ColumnMode>,
-    column_list_expr: BSNode,
-    column_list: RefCell<Vec<String>>,
+    default_sort_column: RefCell<Result<SortSpec, String>>,
+    column_filter_expr: BSNode,
+    column_filter: RefCell<Result<Filter, String>>,
     row_filter_expr: BSNode,
-    row_filter: RefCell<Vec<String>>,
-    editable_expr: BSNode,
-    editable: RefCell<EditMode>,
+    row_filter: RefCell<Result<Filter, String>>,
+    column_editable_expr: BSNode,
+    column_editable: RefCell<Result<Filter, String>>,
     on_select: Rc<RefCell<BSNode>>,
     on_activate: Rc<RefCell<BSNode>>,
     on_edit: Rc<RefCell<BSNode>>,
@@ -131,113 +238,6 @@ fn compare_row(col: i32, m: &TreeModel, r0: &TreeIter, r1: &TreeIter) -> Orderin
     }
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-enum SortDir {
-    Ascending,
-    Descending,
-}
-
-impl SortDir {
-    fn new(v: Value) -> Self {
-        match v {
-            Value::String(c) if &*c == "ascending" => SortDir::Ascending,
-            _ => SortDir::Descending,
-        }
-    }
-}
-
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-enum ColumnMode {
-    Auto,
-    Hide,
-    Exactly,
-}
-
-impl ColumnMode {
-    fn new(v: Value) -> Self {
-        match v {
-            Value::String(c) if &*c == "hide" => ColumnMode::Hide,
-            Value::String(c) if &*c == "exactly" => ColumnMode::Exactly,
-            _ => ColumnMode::Auto,
-        }
-    }
-}
-
-#[derive(Debug, Clone, PartialEq, Eq)]
-enum EditMode {
-    Full(bool),
-    Partial(HashSet<String>),
-}
-
-impl EditMode {
-    fn new(v: Value) -> Self {
-        match v {
-            Value::True => EditMode::Full(true),
-            Value::False => EditMode::Full(false),
-            Value::String(s) => match serde_json::from_str::<HashSet<String>>(&*s) {
-                Ok(cols) => EditMode::Partial(cols),
-                Err(_) => EditMode::Partial(
-                    split_escaped(&*s, '\\', ',')
-                        .map(|s| String::from(s.trim()))
-                        .collect(),
-                ),
-            },
-            _ => EditMode::Full(false),
-        }
-    }
-}
-
-fn cols_from_val(v: Value) -> Vec<String> {
-    match v {
-        Value::String(c) => match serde_json::from_str::<Vec<String>>(&*c) {
-            Ok(cols) => cols,
-            Err(_) => {
-                split_escaped(&*c, '\\', ',').map(|s| String::from(s.trim())).collect()
-            }
-        },
-        _ => vec![],
-    }
-}
-
-fn apply_spec(
-    mode: ColumnMode,
-    cols: &[String],
-    row_filter: Vec<String>,
-    descr: &mut resolver::Table,
-) {
-    fn filter_cols(
-        cols: &[String],
-        descr: &mut resolver::Table,
-        f: impl Fn(bool) -> bool,
-    ) {
-        let set: HashSet<Path> = HashSet::from_iter(cols.iter().cloned().map(Path::from));
-        let mut i = 0;
-        while i < descr.cols.len() {
-            if f(set.contains(&descr.cols[i].0)) {
-                descr.cols.swap_remove(i);
-            } else {
-                i += 1;
-            }
-        }
-    }
-    match mode {
-        ColumnMode::Hide => filter_cols(cols, descr, |x| x),
-        ColumnMode::Exactly => {
-            let order: HashMap<Path, usize> = HashMap::from_iter(
-                cols.iter().cloned().map(Path::from).enumerate().map(|(i, c)| (c, i)),
-            );
-            filter_cols(&cols, descr, |x| !x);
-            descr.cols.sort_by(|(c0, _), (c1, _)| order[c0].cmp(&order[c1]));
-        }
-        _ => (),
-    }
-    let row_filter: HashSet<String> = row_filter.into_iter().collect();
-    descr.rows.retain(|row| match Path::basename(&row) {
-        None => true,
-        Some(row) => !row_filter.contains(row),
-    })
-}
-
 impl clone::Downgrade for RaeifiedTable {
     type Weak = RaeifiedTableWeak;
 
@@ -259,30 +259,51 @@ impl RaeifiedTable {
         ctx: BSCtx,
         root: ScrolledWindow,
         path: Path,
-        default_sort_column: Option<String>,
-        default_sort_column_direction: SortDir,
-        column_mode: ColumnMode,
-        column_list: Vec<String>,
-        row_filter: Vec<String>,
-        editable: EditMode,
+        default_sort_column: Result<SortSpec, String>,
+        column_filter: Result<Filter, String>,
+        row_filter: Result<Filter, String>,
+        column_editable: Result<Filter, String>,
         on_select: Rc<RefCell<BSNode>>,
         on_activate: Rc<RefCell<BSNode>>,
         on_edit: Rc<RefCell<BSNode>>,
         mut descriptor: resolver::Table,
         selected_path: Label,
     ) -> RaeifiedTable {
-        apply_spec(column_mode, &column_list, row_filter, &mut descriptor);
+        let mut filter_errors = Vec::new();
+        let row_filter = row_filter.unwrap_or_else(|e| {
+            filter_errors.push(format!("invalid row filter: {}", e));
+            Filter::All
+        });
+        let column_filter = column_filter.unwrap_or_else(|e| {
+            filter_errors.push(format!("invalid column filter {}", e));
+            Filter::Auto
+        });
+        let column_editable = column_editable.unwrap_or_else(|e| {
+            filter_errors.push(format!("invalid column editable {}", e));
+            Filter::None
+        });
+        let default_sort_column = default_sort_column.unwrap_or_else(|e| {
+            filter_errors.push(format!("invalid default sort column {}", e));
+            SortSpec::None
+        });
+        descriptor.cols.sort_by_key(|(p, _)| p.clone());
+        descriptor.rows.sort();
+        descriptor.rows.retain(|row| match Path::basename(&row) {
+            None => true,
+            Some(row) => row_filter.is_match(row),
+        });
+        match column_filter {
+            Filter::Auto => {
+                let nrows = (descriptor.rows.len() >> 1) as u64;
+                descriptor.cols.retain(|(_, i)| i.0 >= nrows)
+            }
+            filter => descriptor.cols.retain(|col| match Path::basename(&col.0) {
+                None => true,
+                Some(col) => filter.is_match(&col),
+            }),
+        }
         let view = TreeView::new();
         root.add(&view);
-        let nrows = descriptor.rows.len();
-        descriptor.rows.sort();
-        match column_mode {
-            ColumnMode::Exactly => (),
-            ColumnMode::Auto | ColumnMode::Hide => {
-                descriptor.cols.sort_by_key(|(p, _)| p.clone());
-                descriptor.cols.retain(|(_, i)| i.0 >= (nrows / 2) as u64);
-            }
-        }
         view.get_selection().set_mode(SelectionMode::None);
         let vector_mode = descriptor.cols.len() == 0;
         let column_types = if vector_mode {
@@ -332,6 +353,7 @@ impl RaeifiedTable {
             column.add_attribute(&cell, "text", 0);
             column.set_sort_column_id(0);
             column.set_sizing(TreeViewColumnSizing::Fixed);
+            column.set_resizable(true);
             column
         });
         for col in 0..ncols {
@@ -344,13 +366,8 @@ impl RaeifiedTable {
                 t.0.descriptor.cols[col].0.clone()
             };
             column.pack_start(&cell, true);
-            match &editable {
-                EditMode::Full(v) => cell.set_property_editable(*v),
-                EditMode::Partial(allowed) => {
-                    if allowed.contains(&*name) {
-                        cell.set_property_editable(true);
-                    }
-                }
+            if column_editable.is_match(&*name) {
+                cell.set_property_editable(true);
             }
             let f = Box::new(clone!(@weak t =>
                 move |c: &TreeViewColumn,
@@ -371,6 +388,7 @@ impl RaeifiedTable {
             });
             column.set_sort_column_id(id);
             column.set_sizing(TreeViewColumnSizing::Fixed);
+            column.set_resizable(true);
             t.view().append_column(&column);
         }
         t.store().connect_sort_column_changed(clone!(@weak t => move |_| {
@@ -414,21 +432,26 @@ impl RaeifiedTable {
                 }));
             }));
         });
-        if let Some(col) = &default_sort_column {
-            let col = Path::from(col.clone());
-            let idx = t.0.descriptor.cols.iter().enumerate().find_map(|(i, (c, _))| {
-                if c == &col {
-                    Some(i + 1)
-                } else {
-                    None
+        match &default_sort_column {
+            SortSpec::None => (),
+            SortSpec::Ascending(col) | SortSpec::Descending(col) => {
+                let col = Path::from(col.clone());
+                let idx =
+                    t.0.descriptor.cols.iter().enumerate().find_map(|(i, (c, _))| {
+                        if c == &col {
+                            Some(i + 1)
+                        } else {
+                            None
+                        }
+                    });
+                let dir = match &default_sort_column {
+                    SortSpec::Ascending(_) => gtk::SortType::Ascending,
+                    SortSpec::Descending(_) => gtk::SortType::Descending,
+                    SortSpec::None => unreachable!(),
+                };
+                if let Some(i) = idx {
+                    t.store().set_sort_column_id(gtk::SortColumn::Index(i as u32), dir)
                 }
-            });
-            let dir = match default_sort_column_direction {
-                SortDir::Ascending => gtk::SortType::Ascending,
-                SortDir::Descending => gtk::SortType::Descending,
-            };
-            if let Some(i) = idx {
-                t.store().set_sort_column_id(gtk::SortColumn::Index(i as u32), dir)
             }
         }
         t.0.root.show_all();
@@ -500,11 +523,6 @@ impl RaeifiedTable {
                 ],
             );
             let root = d.get_content_area();
-            let cb_lbl = gtk::Label::new(Some("Type:"));
-            let cb = gtk::ComboBoxText::new();
-            let cb_box = gtk::Box::new(gtk::Orientation::Horizontal, 5);
-            cb_box.add(&cb_lbl);
-            cb_box.add(&cb);
             let data_lbl = gtk::Label::new(Some("value:"));
             let data = gtk::Entry::new();
             let data_box = gtk::Box::new(gtk::Orientation::Horizontal, 5);
@@ -514,51 +532,29 @@ impl RaeifiedTable {
             let err_attrs = pango::AttrList::new();
             err_attrs.insert(pango::Attribute::new_foreground(0xFFFFu16, 0, 0).unwrap());
             err.set_attributes(Some(&err_attrs));
-            for typ in Typ::all() {
-                let name = typ.name();
-                cb.append(Some(name), name);
-            }
-            let typ =
-                Rc::new(RefCell::new(val.borrow().as_ref().and_then(|v| Typ::get(v))));
-            let parse_val = Rc::new(clone!(
-                @strong typ,
-                @strong val,
-                @strong data,
-                @strong err => move || {
-                match &*typ.borrow() {
-                    None => { *val.borrow_mut() = Some(Value::Null); }
-                    Some(typ) => match typ.parse(&*data.get_text()) {
-                        Err(e) => {
-                            *val.borrow_mut() = None;
-                            err.set_text(&format!("{}", e));
-                            data.set_icon_from_icon_name(
-                                gtk::EntryIconPosition::Secondary,
-                                Some("dialog-error")
-                            );
-                        }
-                        Ok(v) => {
-                            err.set_text("");
-                            data.set_icon_from_icon_name(
-                                gtk::EntryIconPosition::Secondary,
-                                None
-                            );
-                            *val.borrow_mut() = Some(v)
-                        }
-                    }
-                }
-            }));
-            cb.set_active_id(typ.borrow().map(|t| t.name()));
-            cb.connect_changed(clone!(@strong typ, @strong parse_val => move |cb| {
-                *typ.borrow_mut() = cb.get_active_id().and_then(|s| {
-                    Typ::from_str(&*s).ok()
-                });
-                parse_val();
-            }));
             if let Some(v) = &*val.borrow() {
-                data.set_text(&format!("{}", v));
+                data.set_text(&format!("{}", WVal(v)));
             }
-            data.connect_changed(clone!(@strong parse_val => move |_| parse_val()));
-            root.add(&cb_box);
+            data.connect_changed(clone!(
+                @strong val,
+                @strong err => move |data| match data.get_text().parse::<Value>() {
+                    Err(e) => {
+                        *val.borrow_mut() = None;
+                        err.set_text(&format!("{}", e));
+                        data.set_icon_from_icon_name(
+                            gtk::EntryIconPosition::Secondary,
+                            Some("dialog-error")
+                        );
+                    }
+                    Ok(v) => {
+                        err.set_text("");
+                        data.set_icon_from_icon_name(
+                            gtk::EntryIconPosition::Secondary,
+                            None
+                        );
+                        *val.borrow_mut() = Some(v)
+                    }
+            }));
             root.add(&data_box);
             root.add(&err);
             root.show_all();
@@ -615,7 +611,6 @@ impl RaeifiedTable {
             let ev = vm::Event::User(LocalEvent::Event(v));
             self.0.on_select.borrow_mut().update(&mut self.0.ctx.borrow_mut(), &ev);
         }
-        self.view().columns_autosize();
     }
 
     pub(super) fn update_subscriptions(&self) {
@@ -675,7 +670,17 @@ impl RaeifiedTable {
                 };
                 let user_r = &self.0.ctx.borrow_mut().user;
                 let s = user_r.backend.subscriber.durable_subscribe(p);
-                s.updates(UpdatesFlags::BEGIN_WITH_LAST, user_r.backend.updates.clone());
+                s.updates(
+                    UpdatesFlags::BEGIN_WITH_LAST | UpdatesFlags::NO_SPURIOUS,
+                    user_r.backend.updates.clone(),
+                );
+                match s.last() {
+                    Event::Unsubscribed => (),
+                    Event::Update(v) => {
+                        let v = format!("{}", WVal(&v)).to_value();
+                        self.0.store.set_value(&row, id as u32, &v);
+                    }
+                }
                 self.0.by_id.borrow_mut().insert(
                     s.id(),
                     Subscription { _sub: s, row: row.clone(), col: id as u32 },
@@ -748,7 +753,7 @@ impl RaeifiedTable {
                     match t.0.update.borrow_mut().pop() {
                         None => break,
                         Some((id, v)) => if let Some(sub) = t.0.by_id.borrow().get(&id) {
-                            let s = &format!("{}", v).to_value();
+                            let s = &format!("{}", WVal(&v)).to_value();
                             t.store().set_value(&sub.row, sub.col, s);
                         }
                     }
@@ -770,7 +775,6 @@ impl RaeifiedTable {
                         }
                         start.next();
                     }
-                    t.view().columns_autosize();
                     Continue(false)
                 }
             }
@@ -784,7 +788,9 @@ impl RaeifiedTable {
         event: &vm::Event<LocalEvent>,
     ) {
         match event {
-            vm::Event::User(_) | vm::Event::Variable(_, _) | vm::Event::Rpc(_, _) => (),
+            vm::Event::User(_) | vm::Event::Variable(_, _, _) | vm::Event::Rpc(_, _) => {
+                ()
+            }
             vm::Event::Netidx(id, value) => {
                 self.0.update.borrow_mut().insert(*id, value.clone());
                 if self.0.update.borrow().len() == 1 {
@@ -798,38 +804,39 @@ impl RaeifiedTable {
 }
 
 impl Table {
-    pub(super) fn new(ctx: BSCtx, spec: view::Table, selected_path: Label) -> Table {
-        let path_expr = BSNode::compile(&mut ctx.borrow_mut(), spec.path);
+    pub(super) fn new(
+        ctx: BSCtx,
+        spec: view::Table,
+        scope: Path,
+        selected_path: Label,
+    ) -> Table {
+        let path_expr = BSNode::compile(&mut ctx.borrow_mut(), scope.clone(), spec.path);
         let path = RefCell::new(path_expr.current().and_then(|v| match v {
             Value::String(path) => Some(Path::from(ArcStr::from(&*path))),
             _ => None,
         }));
-        let default_sort_column_expr =
-            BSNode::compile(&mut ctx.borrow_mut(), spec.default_sort_column);
-        let default_sort_column =
-            RefCell::new(default_sort_column_expr.current().and_then(|v| match v {
-                Value::String(c) => Some(String::from(&*c)),
-                _ => None,
-            }));
-        let default_sort_column_direction_expr =
-            BSNode::compile(&mut ctx.borrow_mut(), spec.default_sort_column_direction);
-        let default_sort_column_direction = RefCell::new(SortDir::new(
-            default_sort_column_direction_expr.current().unwrap_or(Value::Null),
+        let default_sort_column_expr = BSNode::compile(
+            &mut ctx.borrow_mut(),
+            scope.clone(),
+            spec.default_sort_column,
+        );
+        let default_sort_column = RefCell::new(SortSpec::new(
+            default_sort_column_expr.current().unwrap_or(Value::Null),
         ));
-        let column_mode_expr = BSNode::compile(&mut ctx.borrow_mut(), spec.column_mode);
-        let column_mode = RefCell::new(ColumnMode::new(
-            column_mode_expr.current().unwrap_or(Value::Null),
+        let column_filter_expr =
+            BSNode::compile(&mut ctx.borrow_mut(), scope.clone(), spec.column_filter);
+        let column_filter = RefCell::new(Filter::new(
+            column_filter_expr.current().unwrap_or(Value::Null),
         ));
-        let column_list_expr = BSNode::compile(&mut ctx.borrow_mut(), spec.column_list);
-        let column_list = RefCell::new(cols_from_val(
-            column_list_expr.current().unwrap_or(Value::Null),
-        ));
-        let row_filter_expr = BSNode::compile(&mut ctx.borrow_mut(), spec.row_filter);
+        let row_filter_expr =
+            BSNode::compile(&mut ctx.borrow_mut(), scope.clone(), spec.row_filter);
         let row_filter =
-            RefCell::new(cols_from_val(row_filter_expr.current().unwrap_or(Value::Null)));
-        let editable_expr = BSNode::compile(&mut ctx.borrow_mut(), spec.editable);
-        let editable =
-            RefCell::new(EditMode::new(editable_expr.current().unwrap_or(Value::Null)));
+            RefCell::new(Filter::new(row_filter_expr.current().unwrap_or(Value::Null)));
+        let column_editable_expr =
+            BSNode::compile(&mut ctx.borrow_mut(), scope.clone(), spec.column_editable);
+        let column_editable = RefCell::new(Filter::new(
+            column_editable_expr.current().unwrap_or(Value::Null),
+        ));
         let state = RefCell::new(match &*path.borrow() {
             None => TableState::Empty,
             Some(path) => {
@@ -837,14 +844,21 @@ impl Table {
                 TableState::Resolving(path.clone())
             }
         });
-        let on_select =
-            Rc::new(RefCell::new(BSNode::compile(&mut ctx.borrow_mut(), spec.on_select)));
+        let on_select = Rc::new(RefCell::new(BSNode::compile(
+            &mut ctx.borrow_mut(),
+            scope.clone(),
+            spec.on_select,
+        )));
         let on_activate = Rc::new(RefCell::new(BSNode::compile(
             &mut ctx.borrow_mut(),
+            scope.clone(),
             spec.on_activate,
         )));
-        let on_edit =
-            Rc::new(RefCell::new(BSNode::compile(&mut ctx.borrow_mut(), spec.on_edit)));
+        let on_edit = Rc::new(RefCell::new(BSNode::compile(
+            &mut ctx.borrow_mut(),
+            scope,
+            spec.on_edit,
+        )));
         Table {
             ctx,
             state,
@@ -857,20 +871,16 @@ impl Table {
             on_edit,
             default_sort_column_expr,
             default_sort_column,
-            default_sort_column_direction_expr,
-            default_sort_column_direction,
-            column_mode_expr,
-            column_mode,
-            column_list_expr,
-            column_list,
+            column_filter_expr,
+            column_filter,
             row_filter_expr,
             row_filter,
-            editable_expr,
-            editable,
+            column_editable_expr,
+            column_editable,
         }
     }
 
-    fn refresh(&self) {
+    fn refresh(&self, ctx: BSCtxRef) {
         let state = &mut *self.state.borrow_mut();
         let path = &*self.path.borrow();
         match state {
@@ -882,7 +892,7 @@ impl Table {
                     *state = TableState::Empty;
                 }
                 Some(path) => {
-                    self.ctx.borrow().user.backend.resolve_table(path.clone());
+                    ctx.user.backend.resolve_table(path.clone());
                     *state = TableState::Resolving(path.clone());
                 }
             },
@@ -909,52 +919,35 @@ impl Table {
             };
             if &*self.path.borrow() != &new_path {
                 *self.path.borrow_mut() = new_path;
-                self.refresh();
+                self.refresh(ctx);
             }
         }
         if let Some(col) = self.default_sort_column_expr.update(ctx, event) {
-            let new_col = match col {
-                Value::String(c) => Some(String::from(&*c)),
-                _ => None,
-            };
+            let new_col = SortSpec::new(col);
             if &*self.default_sort_column.borrow() != &new_col {
                 *self.default_sort_column.borrow_mut() = new_col;
-                self.refresh();
+                self.refresh(ctx);
             }
         }
-        if let Some(dir) = self.default_sort_column_direction_expr.update(ctx, event) {
-            let new_dir = SortDir::new(dir);
-            if &*self.default_sort_column_direction.borrow() != &new_dir {
-                *self.default_sort_column_direction.borrow_mut() = new_dir;
-                self.refresh();
-            }
-        }
-        if let Some(mode) = self.column_mode_expr.update(ctx, event) {
-            let new_mode = ColumnMode::new(mode);
-            if &*self.column_mode.borrow() != &new_mode {
-                *self.column_mode.borrow_mut() = new_mode;
-                self.refresh();
-            }
-        }
-        if let Some(cols) = self.column_list_expr.update(ctx, event) {
-            let new_lst = cols_from_val(cols);
-            if &*self.column_list.borrow() != &new_lst {
-                *self.column_list.borrow_mut() = new_lst;
-                self.refresh();
+        if let Some(mode) = self.column_filter_expr.update(ctx, event) {
+            let new_filter = Filter::new(mode);
+            if &*self.column_filter.borrow() != &new_filter {
+                *self.column_filter.borrow_mut() = new_filter;
+                self.refresh(ctx);
             }
         }
         if let Some(row_filter) = self.row_filter_expr.update(ctx, event) {
-            let new_lst = cols_from_val(row_filter);
-            if &*self.row_filter.borrow() != &new_lst {
-                *self.row_filter.borrow_mut() = new_lst;
-                self.refresh();
+            let new_filter = Filter::new(row_filter);
+            if &*self.row_filter.borrow() != &new_filter {
+                *self.row_filter.borrow_mut() = new_filter;
+                self.refresh(ctx);
             }
         }
-        if let Some(v) = self.editable_expr.update(ctx, event) {
-            let new_mode = EditMode::new(v);
-            if &*self.editable.borrow() != &new_mode {
-                *self.editable.borrow_mut() = new_mode;
-                self.refresh();
+        if let Some(v) = self.column_editable_expr.update(ctx, event) {
+            let new_ed = Filter::new(v);
+            if &*self.column_editable.borrow() != &new_ed {
+                *self.column_editable.borrow_mut() = new_ed;
+                self.refresh(ctx);
             }
         }
         self.on_activate.borrow_mut().update(ctx, event);
@@ -967,7 +960,7 @@ impl Table {
         match event {
             vm::Event::Netidx(_, _)
             | vm::Event::Rpc(_, _)
-            | vm::Event::Variable(_, _)
+            | vm::Event::Variable(_, _, _)
             | vm::Event::User(LocalEvent::Event(_)) => (),
             vm::Event::User(LocalEvent::TableResolved(path, descriptor)) => {
                 let state = &mut *self.state.borrow_mut();
@@ -980,11 +973,9 @@ impl Table {
                                 self.root.clone(),
                                 path.clone(),
                                 self.default_sort_column.borrow().clone(),
-                                self.default_sort_column_direction.borrow().clone(),
-                                self.column_mode.borrow().clone(),
-                                self.column_list.borrow().clone(),
+                                self.column_filter.borrow().clone(),
                                 self.row_filter.borrow().clone(),
-                                self.editable.borrow().clone(),
+                                self.column_editable.borrow().clone(),
                                 self.on_select.clone(),
                                 self.on_activate.clone(),
                                 self.on_edit.clone(),
